package com.clx.performance.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.clx.performance.dao.IntegralStatisticsDao;
import com.clx.performance.dao.IntegralTruckDao;
import com.clx.performance.enums.PerformanceResultEnum;
import com.clx.performance.extranal.user.DriverService;
import com.clx.performance.model.IntegralRecord;
import com.clx.performance.model.IntegralStatistics;
import com.clx.performance.model.IntegralTruck;
import com.clx.performance.param.pc.IntegralTruckAddParam;
import com.clx.performance.param.pc.PageIntegralTruckParam;
import com.clx.performance.service.IntegralMqService;
import com.clx.performance.service.IntegralRecordService;
import com.clx.performance.service.IntegralStatisticsService;
import com.clx.performance.service.IntegralTruckService;
import com.clx.performance.utils.LocalDateTimeUtils;
import com.clx.performance.vo.pc.IntegralTruckVO;
import com.clx.user.vo.feign.DriverInfoFeignVo;
import com.msl.common.exception.ServiceSystemException;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@Service
@AllArgsConstructor
public class IntegralTruckServiceImpl implements IntegralTruckService {

    private final IntegralTruckDao integralTruckDao;
    private final IntegralStatisticsDao integralStatisticsDao;

    private final IntegralRecordService integralRecordService;
    private final IntegralStatisticsService integralStatisticsService;

    private final DriverService driverService;
    private final IntegralMqService integralMqService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveTruck(IntegralTruckAddParam param) {
        String truckNo = param.getTruckNo();

        boolean existFlag = integralTruckDao.existByTruckNo(truckNo);
        if (existFlag) {throw new ServiceSystemException(PerformanceResultEnum.INTEGRAL_TRUCK_DRIVER_EXIST);}

        IntegralTruck truck = new IntegralTruck();
        truck.setTruckNo(truckNo);
        truck.setDriverName(param.getDriverName());
        truck.setDriverMobile(param.getDriverMobile());
        truck.setEchelon(1);

        String statisticsDate = LocalDateTimeUtils.getWeekStart();
        int protectionIntegral = integralStatisticsService.protectionIntegralCalc(statisticsDate);
        int baseIntegral = 100;

        // 初始化统计
        IntegralStatistics statistics = new IntegralStatistics();
        statistics.setTruckNo(truckNo);
        statistics.setStatisticsDate(statisticsDate);
        statistics.setIntegralBase(0);
        statistics.setIntegral(protectionIntegral+baseIntegral);   //保护分+基础分
        statistics.setRank(1);
        integralStatisticsDao.saveEntity(statistics);

        IntegralStatistics statisticsNext = new IntegralStatistics();
        statisticsNext.setTruckNo(truckNo);
        statisticsNext.setStatisticsDate(LocalDateTimeUtils.formatTime(LocalDateTimeUtils.parseTime(statistics.getStatisticsDate()).plusDays(7)));
        statisticsNext.setIntegralBase(0);
        statisticsNext.setIntegral(0);
        statisticsNext.setRank(1);
        integralStatisticsDao.saveEntity(statisticsNext);

        // 新增车辆
        truck.setIntegral(statistics.getIntegral());
        integralTruckDao.saveEntity(truck);

        // 新增积分记录
        integralRecordService.save(statistics.getId(), statistics.getStatisticsDate(), statistics.getTruckNo(), IntegralRecord.type.PROTECTION.getValue(), protectionIntegral, "新用户");
        integralRecordService.save(statistics.getId(), statistics.getStatisticsDate(), statistics.getTruckNo(), IntegralRecord.type.BASE.getValue(), baseIntegral, "基础分");

        // 发送mq (车辆新增)
        integralMqService.truckAdd(statistics.getId(), param.getTruckNo());

    }

    @Override
    public IPage<IntegralTruckVO> pageTruckList(PageIntegralTruckParam param) {
        return integralTruckDao.pageByParam(param);
    }

    @Override
    public IntegralTruckVO getTruckDriverInfo(String truckNo) {
        IntegralTruckVO truck = null;

        // 查询车辆信息
        DriverInfoFeignVo driverTruckInfoFeignVo = driverService.getTruckOwnInfo(truckNo).get();

        if (null != driverTruckInfoFeignVo) {
            truck = new IntegralTruckVO();
            truck.setTruckNo(truckNo);
            truck.setDriverName(driverTruckInfoFeignVo.getName());
            truck.setDriverMobile(driverTruckInfoFeignVo.getMobile());
        }

        return truck;
    }

    @Override
    public Integer getTruckCount(Integer truckEchelon) {
        int count = (int)integralTruckDao.countByTruckEchelon(truckEchelon);

        return count;
    }

}
